@@ -17,13 +17,13 @@
 
 #include "./arrow_types.h"
 
-#if defined(ARROW_R_WITH_ARROW)
 #include <arrow/ipc/reader.h>
 #include <arrow/ipc/writer.h>
 
 // [[arrow::export]]
-int64_t ipc___Message__body_length(const std::unique_ptr<arrow::ipc::Message>& message) {
-  return message->body_length();
+r_vec_size ipc___Message__body_length(
+    const std::unique_ptr<arrow::ipc::Message>& message) {
+  return r_vec_size(message->body_length());
 }
 
 // [[arrow::export]]
@@ -39,8 +39,8 @@ std::shared_ptr<arrow::Buffer> ipc___Message__body(
 }
 
 // [[arrow::export]]
-int64_t ipc___Message__Verify(const std::unique_ptr<arrow::ipc::Message>& message) {
-  return message->Verify();
+r_vec_size ipc___Message__Verify(const std::unique_ptr<arrow::ipc::Message>& message) {
+  return r_vec_size(message->Verify());
 }
 
 // [[arrow::export]]
@@ -61,8 +61,9 @@ std::shared_ptr<arrow::RecordBatch> ipc___ReadRecordBatch__Message__Schema(
     const std::shared_ptr<arrow::Schema>& schema) {
   // TODO: perhaps this should come from the R side
   arrow::ipc::DictionaryMemo memo;
-  return ValueOrStop(arrow::ipc::ReadRecordBatch(*message, schema, &memo,
-                                                 arrow::ipc::IpcReadOptions::Defaults()));
+  auto batch = ValueOrStop(arrow::ipc::ReadRecordBatch(
+      *message, schema, &memo, arrow::ipc::IpcReadOptions::Defaults()));
+  return batch;
 }
 
 // [[arrow::export]]
@@ -85,7 +86,8 @@ std::shared_ptr<arrow::Schema> ipc___ReadSchema_Message(
 // [[arrow::export]]
 std::shared_ptr<arrow::ipc::MessageReader> ipc___MessageReader__Open(
     const std::shared_ptr<arrow::io::InputStream>& stream) {
-  return arrow::ipc::MessageReader::Open(stream);
+  return std::shared_ptr<arrow::ipc::MessageReader>(
+      arrow::ipc::MessageReader::Open(stream));
 }
 
 // [[arrow::export]]
@@ -99,5 +101,3 @@ std::shared_ptr<arrow::ipc::Message> ipc___ReadMessage(
     const std::shared_ptr<arrow::io::InputStream>& stream) {
   return ValueOrStop(arrow::ipc::ReadMessage(stream.get()));
 }
-
-#endif
