@@ -63,9 +63,7 @@ using parquet::LogicalType;
 
 using parquet::internal::LevelInfo;
 
-namespace parquet {
-
-namespace arrow {
+namespace parquet::arrow {
 
 // ----------------------------------------------------------------------
 // Parquet to Arrow schema conversion
@@ -841,7 +839,7 @@ std::function<std::shared_ptr<::arrow::DataType>(FieldVector)> GetNestedFactory(
   switch (inferred_type.id()) {
     case ::arrow::Type::STRUCT:
       if (origin_type.id() == ::arrow::Type::STRUCT) {
-        return ::arrow::struct_;
+        return [](FieldVector fields) { return ::arrow::struct_(std::move(fields)); };
       }
       break;
     case ::arrow::Type::LIST:
@@ -1106,5 +1104,4 @@ Status SchemaManifest::Make(const SchemaDescriptor* schema,
   return Status::OK();
 }
 
-}  // namespace arrow
-}  // namespace parquet
+}  // namespace parquet::arrow
