@@ -15,33 +15,56 @@
 # specific language governing permissions and limitations
 # under the License.
 
-args <- commandArgs(TRUE)
-VERSION <- args[1]
-dst_dir <- paste0("libarrow/arrow-", VERSION)
-
-# TESTING is set in test-nixlibs.R; it won't be set when called from configure
-test_mode <- exists("TESTING")
-
-# Prevent error with binary selection during testing.
-if (test_mode && is.na(VERSION)) {
-  VERSION <- "8.0.0.9000"
-}
-
-dev_version <- package_version(VERSION)[1, 4]
-
-# Small dev versions are added for R-only changes during CRAN submission.
-if (is.na(dev_version) || dev_version < "100") {
-  VERSION <- package_version(VERSION)[1, 1:3]
-  arrow_repo <- paste0(getOption("arrow.repo", sprintf("https://apache.jfrog.io/artifactory/arrow/r/%s", VERSION)), "/libarrow/")
-} else {
-  arrow_repo <- paste0(getOption("arrow.dev_repo", "https://nightlies.apache.org/arrow/r"), "/libarrow/")
-}
-
-options(.arrow.cleanup = character()) # To collect dirs to rm on exit
-on.exit(unlink(getOption(".arrow.cleanup")))
-
+#### Fuctions #### check end of file for main logic
 env_is <- function(var, value) identical(tolower(Sys.getenv(var)), value)
 
+# Log messages in the style of the configure script
+lg <- function(..., .indent = "***") {
+  cat(.indent, " ", sprintf(...), "\n", sep = "")
+}
+
+cleanup <- function(path) {
+  options(.arrow.cleanup = c(getOption(".arrow.cleanup"), path))
+}
+
+# Exit the script after logging with .status=1 instead of throwing an error
+exit <- function(..., .status = 1) {
+  lg(...)
+  q(save = "no", status = .status)
+}
+
+
+# checks the nightly repo for the latest nightly version X.Y.Z.100<dev>
+find_latest_nightly <- function(description_version) {
+  if (!startsWith(arrow_repo, "https://nightlies.apache.org/arrow/r")) {
+    lg("Detected non standard dev repo: %s, not checking latest nightly version.", arrow_repo)
+    return(description_version)
+  }
+
+  res <- try(
+    {
+      # Binaries are only uploaded if all jobs pass so can just look at the source versions.
+      urls <- readLines("https://nightlies.apache.org/arrow/r/src/contrib")
+      versions <- grep("arrow_.*\\.tar\\.gz", urls, value = TRUE)
+      versions <- sub(".*arrow_(.*)\\.tar\\.gz.*", "\\1", x = versions)
+      versions <- sapply(versions, package_version)
+      versions <- data.frame(do.call(rbind, versions))
+      matching_major <- versions[versions$X1 == description_version[1, 1], ]
+      latest <- matching_major[which.max(matching_major$X4), ]
+      package_version(paste0(latest, collapse = "."))
+    },
+    silent = quietly
+  )
+  if (inherits(res, "try-error")) {
+    lg("Failed to find latest nightly for %s", description_version)
+    latest <- description_version
+  } else {
+    lg("Found latest nightly for %s: %s", description_version, res)
+    latest <- res
+  }
+  latest
+}
+
 try_download <- function(from_url, to_file, hush = quietly) {
   status <- try(
     suppressWarnings(
@@ -53,52 +76,59 @@ try_download <- function(from_url, to_file, hush = quietly) {
   !inherits(status, "try-error") && status == 0
 }
 
-not_cran <- env_is("NOT_CRAN", "true")
-if (not_cran) {
-  # Set more eager defaults
-  if (env_is("LIBARROW_BINARY", "")) {
-    Sys.setenv(LIBARROW_BINARY = "true")
-  }
-  if (env_is("LIBARROW_MINIMAL", "")) {
-    Sys.setenv(LIBARROW_MINIMAL = "false")
-  }
-}
-
-# For local debugging, set ARROW_R_DEV=TRUE to make this script print more
-quietly <- !env_is("ARROW_R_DEV", "true")
-
-# The default will build from source as a fallback if a binary is not found or shouldn't be used
-# Set LIBARROW_BUILD=FALSE to ensure that we use a previously built libarrow
-# and don't fall back to a full source build
-build_ok <- !env_is("LIBARROW_BUILD", "false")
-
-# Check if we're authorized to download (not asked an offline build).
-# (Note that cmake will still be downloaded if necessary
-#  https://arrow.apache.org/docs/developers/cpp/building.html#offline-builds)
-download_ok <- !test_mode && !env_is("TEST_OFFLINE_BUILD", "true")
-
-# This "tools/thirdparty_dependencies" path, within the tar file, might exist if
-# create_package_with_all_dependencies() was run, or if someone has created it
-# manually before running make build.
-# If you change this path, you also need to edit
-# `create_package_with_all_dependencies()` in install-arrow.R
-thirdparty_dependency_dir <- Sys.getenv("ARROW_THIRDPARTY_DEPENDENCY_DIR", "tools/thirdparty_dependencies")
-
-
 download_binary <- function(lib) {
-  libfile <- tempfile()
+  libfile <- paste0("arrow-", VERSION, ".zip")
   binary_url <- paste0(arrow_repo, "bin/", lib, "/arrow-", VERSION, ".zip")
   if (try_download(binary_url, libfile)) {
     if (!quietly) {
-      cat(sprintf("*** Successfully retrieved C++ binaries (%s)\n", lib))
+      lg("Successfully retrieved C++ binaries (%s)", lib)
     }
   } else {
     if (!quietly) {
-      cat(sprintf("*** Downloading libarrow binary failed for version %s (%s)\n    at %s\n",
-                  VERSION, lib, binary_url))
+      lg(
+        "Downloading libarrow binary failed for version %s (%s)\n    at %s",
+        VERSION, lib, binary_url
+      )
     }
     libfile <- NULL
   }
+  # Explicitly setting the env var to "false" will skip checksum validation
+  # e.g. in case the included checksums are stale.
+  skip_checksum <- env_is("ARROW_R_ENFORCE_CHECKSUM", "false")
+  enforce_checksum <- env_is("ARROW_R_ENFORCE_CHECKSUM", "true")
+  checksum_path <- Sys.getenv("ARROW_R_CHECKSUM_PATH", "tools/checksums")
+  # validate binary checksum for CRAN release only
+  if (!skip_checksum && dir.exists(checksum_path) && is_release ||
+    enforce_checksum) {
+    checksum_file <- sub(".+/bin/(.+\\.zip)", "\\1\\.sha512", binary_url)
+    checksum_file <- file.path(checksum_path, checksum_file)
+    checksum_cmd <- "shasum"
+    checksum_args <- c("--status", "-a", "512", "-c", checksum_file)
+
+    # shasum is not available on all linux versions
+    status_shasum <- try(
+      suppressWarnings(
+        system2("shasum", args = c("--help"), stdout = FALSE, stderr = FALSE)
+      ),
+      silent = TRUE
+    )
+
+    if (inherits(status_shasum, "try-error") || is.integer(status_shasum) && status_shasum != 0) {
+      checksum_cmd <- "sha512sum"
+      checksum_args <- c("--status", "-c", checksum_file)
+    }
+
+    checksum_ok <- system2(checksum_cmd, args = checksum_args)
+
+    if (checksum_ok != 0) {
+      cat("*** Checksum validation failed for libarrow binary: ", libfile, "\n")
+      unlink(libfile)
+      libfile <- NULL
+    } else {
+      cat("*** Checksum validated successfully for libarrow binary: ", libfile, "\n")
+    }
+  }
+
   libfile
 }
 
@@ -114,9 +144,17 @@ download_binary <- function(lib) {
 #    * "linux-openssl-1.0" (OpenSSL 1.0)
 #    * "linux-openssl-1.1" (OpenSSL 1.1)
 #    * "linux-openssl-3.0" (OpenSSL 3.0)
+#    * "macos-amd64-openssl-1.1" (OpenSSL 1.1)
+#    * "macos-amd64-openssl-3.0" (OpenSSL 3.0)
+#    * "macos-arm64-openssl-1.1" (OpenSSL 1.1)
+#    * "macos-arm64-openssl-3.0" (OpenSSL 3.0)
 #   These string values, along with `NULL`, are the potential return values of
 #   this function.
 identify_binary <- function(lib = Sys.getenv("LIBARROW_BINARY"), info = distro()) {
+  if (on_windows) {
+    return("windows")
+  }
+
   lib <- tolower(lib)
   if (identical(lib, "")) {
     # Not specified. Check the allowlist.
@@ -125,16 +163,16 @@ identify_binary <- function(lib = Sys.getenv("LIBARROW_BINARY"), info = distro()
 
   if (identical(lib, "false")) {
     # Do not download a binary
-    NULL
+    lib <- NULL
   } else if (!identical(lib, "true")) {
     # Env var provided an os-version to use, to override our logic.
     # We don't validate that this exists. If it doesn't, the download will fail
     # and the build will fall back to building from source
-    lib
   } else {
     # See if we can find a suitable binary
-    select_binary()
+    lib <- select_binary()
   }
+  lib
 }
 
 check_allowlist <- function(os, allowed = "https://raw.githubusercontent.com/apache/arrow/main/r/tools/nixlibs-allowlist.txt") {
@@ -142,7 +180,7 @@ check_allowlist <- function(os, allowed = "https://raw.githubusercontent.com/apa
     # Try a remote allowlist so that we can add/remove without a release
     suppressWarnings(readLines(allowed)),
     # Fallback to default: allowed only on Ubuntu and CentOS/RHEL
-    error = function(e) c("ubuntu", "centos", "redhat", "rhel")
+    error = function(e) c("ubuntu", "centos", "redhat", "rhel", "darwin")
   )
   # allowlist should contain valid regular expressions (plain strings ok too)
   any(grepl(paste(allowlist, collapse = "|"), os))
@@ -151,34 +189,39 @@ check_allowlist <- function(os, allowed = "https://raw.githubusercontent.com/apa
 select_binary <- function(os = tolower(Sys.info()[["sysname"]]),
                           arch = tolower(Sys.info()[["machine"]]),
                           test_program = test_for_curl_and_openssl) {
-  if (identical(os, "linux") && identical(arch, "x86_64")) {
-    # We only host x86 linux binaries today
-    tryCatch(
+  if (identical(os, "darwin") || (identical(os, "linux") && identical(arch, "x86_64"))) {
+    # We only host x86 linux binaries and x86 & arm64 macos today
+    binary <- tryCatch(
       # Somehow the test program system2 call errors on the sanitizer builds
       # so globally handle the possibility that this could fail
       {
         errs <- compile_test_program(test_program)
-        determine_binary_from_stderr(errs)
+        openssl_version <- determine_binary_from_stderr(errs)
+        arch <- ifelse(identical(os, "darwin"), paste0("-", arch, "-"), "-")
+        ifelse(is.null(openssl_version), NULL, paste0(os, arch, openssl_version))
       },
       error = function(e) {
-        cat("*** Unable to find libcurl and openssl\n")
+        lg("Unable to find libcurl and openssl")
         NULL
       }
     )
   } else {
     # No binary available for arch
-    cat(sprintf("*** Building on %s %s\n", os, arch))
-    NULL
+    lg("Building on %s %s", os, arch)
+    binary <- NULL
   }
+  return(binary)
 }
 
 # This tests that curl and OpenSSL are present (bc we can include their headers)
 # and it checks for other versions/features and raises errors that we grep for
 test_for_curl_and_openssl <- "
+#ifndef __APPLE__
 #include <ciso646>
 #ifdef _LIBCPP_VERSION
 #error Using libc++
 #endif
+#endif
 
 #include <curl/curl.h>
 #include <openssl/opensslv.h>
@@ -194,41 +237,72 @@ test_for_curl_and_openssl <- "
 "
 
 compile_test_program <- function(code) {
-  # Note: if we wanted to check for openssl on macOS, we'd have to set the brew
-  # path as a -I directory. But since we (currently) only run this code to
-  # determine whether we can download a Linux binary, it's not relevant.
-  runner <- "`R CMD config CXX17` `R CMD config CPPFLAGS` `R CMD config CXX17FLAGS` `R CMD config CXX17STD` -E -xc++"
+  openssl_dir <- ""
+  if (on_macos) {
+    openssl_root_dir <- get_macos_openssl_dir()
+    openssl_dir <- paste0("-I", openssl_root_dir, "/include")
+  }
+  runner <- paste(
+    R_CMD_config("CXX17"),
+    openssl_dir,
+    R_CMD_config("CPPFLAGS"),
+    R_CMD_config("CXX17FLAGS"),
+    R_CMD_config("CXX17STD"),
+    "-E",
+    "-xc++"
+  )
   suppressWarnings(system2("echo", sprintf('"%s" | %s -', code, runner), stdout = FALSE, stderr = TRUE))
 }
 
+get_macos_openssl_dir <- function() {
+  openssl_root_dir <- Sys.getenv("OPENSSL_ROOT_DIR", NA)
+  header <- "openssl/opensslv.h"
+  if (is.na(openssl_root_dir) || !file.exists(file.path(openssl_root_dir, "include", header))) {
+    # try to guess default openssl include dir based on CRAN's build script
+    # https://github.com/R-macos/recipes/blob/master/build.sh#L35
+    if (identical(Sys.info()["machine"], "arm64") && file.exists(file.path("/opt/R/arm64/include", header))) {
+      openssl_root_dir <- "/opt/R/arm64"
+    } else if (identical(Sys.info()["machine"], "x86_64") && file.exists(file.path("/opt/R/x86_64/include", header))) {
+      openssl_root_dir <- "/opt/R/x86_64"
+    } else {
+      openssl_root_dir <- "/usr/local"
+    }
+  }
+  return(openssl_root_dir)
+}
+
 # (built with newer devtoolset but older glibc (2.17) for broader compatibility,# like manylinux2014)
 determine_binary_from_stderr <- function(errs) {
   if (is.null(attr(errs, "status"))) {
     # There was no error in compiling: so we found libcurl and OpenSSL >= 1.1,
     # openssl is < 3.0
-    cat("*** Found libcurl and OpenSSL >= 1.1\n")
-    return("linux-openssl-1.1")
+    lg("Found libcurl and OpenSSL >= 1.1")
+    return("openssl-1.1")
     # Else, check for dealbreakers:
-  } else if (any(grepl("Using libc++", errs, fixed = TRUE))) {
-    # Our binaries are all built with GNU stdlib so they fail with libc++
-    cat("*** Found libc++\n")
+  } else if (!on_macos && any(grepl("Using libc++", errs, fixed = TRUE))) {
+    # Our linux binaries are all built with GNU stdlib so they fail with libc++
+    lg("Linux binaries incompatible with libc++")
     return(NULL)
   } else if (header_not_found("curl/curl", errs)) {
-    cat("*** libcurl not found\n")
+    lg("libcurl not found")
     return(NULL)
   } else if (header_not_found("openssl/opensslv", errs)) {
-    cat("*** OpenSSL not found\n")
+    lg("OpenSSL not found")
     return(NULL)
   } else if (any(grepl("OpenSSL version too old", errs))) {
-    cat("*** OpenSSL found but version >= 1.0.2 is required for some features\n")
+    lg("OpenSSL found but version >= 1.0.2 is required for some features")
     return(NULL)
     # Else, determine which other binary will work
   } else if (any(grepl("Using OpenSSL version 1.0", errs))) {
-    cat("*** Found libcurl and OpenSSL < 1.1\n")
-    return("linux-openssl-1.0")
+    if (on_macos) {
+      lg("OpenSSL 1.0 is not supported on macOS")
+      return(NULL)
+    }
+    lg("Found libcurl and OpenSSL < 1.1")
+    return("openssl-1.0")
   } else if (any(grepl("Using OpenSSL version 3", errs))) {
-    cat("*** Found libcurl and OpenSSL >= 3.0.0\n")
-    return("linux-openssl-3.0")
+    lg("Found libcurl and OpenSSL >= 3.0.0")
+    return("openssl-3.0")
   }
   NULL
 }
@@ -241,6 +315,11 @@ header_not_found <- function(header, errs) {
 #### start distro ####
 
 distro <- function() {
+  # This is not part of distro but needed to enable prebuilt binaries on macos
+  if (on_macos) {
+    return(list(id = "darwin", arch = tolower(Sys.info()[["machine"]])))
+  }
+
   # The code in this script is a (potentially stale) copy of the distro package
   if (requireNamespace("distro", quietly = TRUE)) {
     # Use the version from the package, which may be updated from this
@@ -351,7 +430,7 @@ find_local_source <- function() {
   )
   for (cpp_dir in cpp_dir_options) {
     if (file.exists(file.path(cpp_dir, "src/arrow/api.h"))) {
-      cat(paste0("*** Found local C++ source: '", cpp_dir, "'\n"))
+      lg("Found local C++ source: '%s'", cpp_dir)
       return(cpp_dir)
     }
   }
@@ -372,7 +451,7 @@ env_vars_as_string <- function(env_var_list) {
   if (nchar(env_var_string) > 30000) {
     # This could happen if the full paths in *_SOURCE_URL were *very* long.
     # A more formal check would look at getconf ARG_MAX, but this shouldn't matter
-    cat("*** Warning: Environment variables are very long. This could cause issues on some shells.\n")
+    lg("Warning: Environment variables are very long. This could cause issues on some shells.")
   }
   env_var_string
 }
@@ -397,7 +476,7 @@ build_libarrow <- function(src_dir, dst_dir) {
     Sys.setenv(MAKEFLAGS = makeflags)
   }
   if (!quietly) {
-    cat("*** Building with MAKEFLAGS=", makeflags, "\n")
+    lg("Building with MAKEFLAGS=", makeflags)
   }
   # Check for libarrow build dependencies:
   # * cmake
@@ -411,7 +490,7 @@ build_libarrow <- function(src_dir, dst_dir) {
     # But normally we'll just build in a tmp dir
     build_dir <- tempfile()
   }
-  options(.arrow.cleanup = c(getOption(".arrow.cleanup"), build_dir))
+  cleanup(build_dir)
 
   env_var_list <- c(
     SOURCE_DIR = src_dir,
@@ -436,7 +515,7 @@ build_libarrow <- function(src_dir, dst_dir) {
 
   dep_source <- Sys.getenv("ARROW_DEPENDENCY_SOURCE")
   if (dep_source %in% c("", "AUTO") && !nzchar(Sys.which("pkg-config"))) {
-    cat("**** pkg-config not installed, setting ARROW_DEPENDENCY_SOURCE=BUNDLED\n")
+    lg("pkg-config not installed, setting ARROW_DEPENDENCY_SOURCE=BUNDLED", .indent = "****")
     env_var_list <- c(env_var_list, ARROW_DEPENDENCY_SOURCE = "BUNDLED")
   }
 
@@ -465,7 +544,7 @@ build_libarrow <- function(src_dir, dst_dir) {
   }
   env_vars <- env_vars_as_string(env_var_list)
 
-  cat("**** arrow", ifelse(quietly, "", paste("with", env_vars)), "\n")
+  lg("arrow %s", ifelse(quietly, "", paste("with", env_vars)), .indent = "****")
 
   build_log_path <- tempfile(fileext = ".log")
   status <- suppressWarnings(system2(
@@ -478,10 +557,12 @@ build_libarrow <- function(src_dir, dst_dir) {
 
   if (status != 0) {
     # It failed :(
-    cat("**** Error building Arrow C++.", "\n")
+    lg("Error building Arrow C++.", .indent = "****")
     if (quietly) {
-      cat("**** Printing contents of build log because the build failed", 
-          "while ARROW_R_DEV was set to FALSE\n")
+      cat(
+        "**** Printing contents of build log because the build failed",
+        "while ARROW_R_DEV was set to FALSE\n"
+      )
       cat(readLines(build_log_path), sep = "\n")
       cat("**** Complete build log may still be present at", build_log_path, "\n")
     }
@@ -494,19 +575,19 @@ ensure_cmake <- function(cmake_minimum_required = "3.16") {
 
   if (is.null(cmake)) {
     # If not found, download it
-    cat("**** cmake\n")
+    lg("cmake", .indent = "****")
     CMAKE_VERSION <- Sys.getenv("CMAKE_VERSION", "3.26.4")
-    if (tolower(Sys.info()[["sysname"]]) %in% "darwin") {
+    if (on_macos) {
       postfix <- "-macos-universal.tar.gz"
     } else if (tolower(Sys.info()[["machine"]]) %in% c("arm64", "aarch64")) {
       postfix <- "-linux-aarch64.tar.gz"
     } else if (tolower(Sys.info()[["machine"]]) == "x86_64") {
       postfix <- "-linux-x86_64.tar.gz"
     } else {
-      stop(paste0(
+      exit(paste0(
         "*** cmake was not found locally.\n",
         "    Please make sure cmake >= ", cmake_minimum_required,
-        " is installed and available on your PATH.\n"
+        " is installed and available on your PATH."
       ))
     }
     cmake_binary_url <- paste0(
@@ -517,7 +598,7 @@ ensure_cmake <- function(cmake_minimum_required = "3.16") {
     cmake_dir <- tempfile()
     download_successful <- try_download(cmake_binary_url, cmake_tar)
     if (!download_successful) {
-      cat(paste0(
+      exit(paste0(
         "*** cmake was not found locally and download failed.\n",
         "    Make sure cmake >= ", cmake_minimum_required,
         " is installed and available on your PATH,\n",
@@ -527,7 +608,7 @@ ensure_cmake <- function(cmake_minimum_required = "3.16") {
     }
     untar(cmake_tar, exdir = cmake_dir)
     unlink(cmake_tar)
-    options(.arrow.cleanup = c(getOption(".arrow.cleanup"), cmake_dir))
+    cleanup(cmake_dir)
     cmake <- paste0(
       cmake_dir,
       "/cmake-", CMAKE_VERSION, sub(".tar.gz", "", postfix, fixed = TRUE),
@@ -536,7 +617,7 @@ ensure_cmake <- function(cmake_minimum_required = "3.16") {
   } else {
     # Show which one we found
     # Full source builds will always show "cmake" in the logs
-    cat(sprintf("**** cmake: %s\n", cmake))
+    lg("cmake: %s", cmake, .indent = "****")
   }
   cmake
 }
@@ -671,6 +752,7 @@ is_feature_requested <- function(env_varname, default = env_is("LIBARROW_MINIMAL
 with_cloud_support <- function(env_var_list) {
   arrow_s3 <- is_feature_requested("ARROW_S3")
   arrow_gcs <- is_feature_requested("ARROW_GCS")
+
   if (arrow_s3 || arrow_gcs) {
     # User wants S3 or GCS support.
     # Make sure that we have curl and openssl system libs
@@ -685,11 +767,6 @@ with_cloud_support <- function(env_var_list) {
       cat("**** ", start_msg, " support ", msg, "; building with ", off_flags, "\n")
     }
 
-    # Check the features
-    # This duplicates what we do with the test program above when we check
-    # capabilities for using binaries. We could consider consolidating this
-    # logic, though these use cmake in order to match exactly what we do in the
-    # libarrow build, and maybe that increases the fidelity.
     if (!cmake_find_package("CURL", NULL, env_var_list)) {
       # curl on macos should be installed, so no need to alter this for macos
       # TODO: check for apt/yum/etc. and message the right thing?
@@ -711,7 +788,7 @@ with_cloud_support <- function(env_var_list) {
 cmake_find_package <- function(pkg, version = NULL, env_var_list) {
   td <- tempfile()
   dir.create(td)
-  options(.arrow.cleanup = c(getOption(".arrow.cleanup"), td))
+  cleanup(td)
   find_package <- paste0("find_package(", pkg, " ", version, " REQUIRED)")
   writeLines(find_package, file.path(td, "CMakeLists.txt"))
   env_vars <- env_vars_as_string(env_var_list)
@@ -726,22 +803,101 @@ cmake_find_package <- function(pkg, version = NULL, env_var_list) {
   system(cmake_cmd, ignore.stdout = TRUE, ignore.stderr = TRUE) == 0
 }
 
-#####
+############### Main logic #############
+args <- commandArgs(TRUE)
+VERSION <- args[1]
 
-if (!test_mode && !file.exists(paste0(dst_dir, "/include/arrow/api.h"))) {
+# TESTING is set in test-nixlibs.R; it won't be set when called from configure
+test_mode <- exists("TESTING")
+
+# Prevent error with binary selection during testing.
+if (test_mode && is.na(VERSION)) {
+  VERSION <- "8.0.0.9000"
+}
+
+VERSION <- package_version(VERSION)
+dev_version <- VERSION[1, 4]
+# Small dev versions are added for R-only changes during CRAN submission
+is_release <- is.na(dev_version) || dev_version < "100"
+
+on_macos <- tolower(Sys.info()[["sysname"]]) == "darwin"
+on_windows <- tolower(Sys.info()[["sysname"]]) == "windows"
+
+# For local debugging, set ARROW_R_DEV=TRUE to make this script print more
+quietly <- !env_is("ARROW_R_DEV", "true")
+
+not_cran <- env_is("NOT_CRAN", "true")
+
+if (is_release) {
+  VERSION <- VERSION[1, 1:3]
+  arrow_repo <- paste0(getOption("arrow.repo", sprintf("https://apache.jfrog.io/artifactory/arrow/r/%s", VERSION)), "/libarrow/")
+} else {
+  not_cran <- TRUE
+  arrow_repo <- paste0(getOption("arrow.dev_repo", "https://nightlies.apache.org/arrow/r"), "/libarrow/")
+  VERSION <- find_latest_nightly(VERSION)
+}
+
+# To collect dirs to rm on exit, use del() to add dirs
+# we reset it to avoid errors on reruns in the same session.
+options(.arrow.cleanup = character())
+on.exit(unlink(getOption(".arrow.cleanup"), recursive = TRUE), add = TRUE)
+
+# enable full featured builds for macOS in case of CRAN source builds.
+if (not_cran || on_macos) {
+  # Set more eager defaults
+  if (env_is("LIBARROW_BINARY", "")) {
+    Sys.setenv(LIBARROW_BINARY = "true")
+  }
+  if (env_is("LIBARROW_MINIMAL", "")) {
+    Sys.setenv(LIBARROW_MINIMAL = "false")
+  }
+}
+
+# The default will build from source as a fallback if a binary is not found or shouldn't be used
+# Set LIBARROW_BUILD=FALSE to ensure that we use a previously built libarrow
+# and don't fall back to a full source build
+build_ok <- !env_is("LIBARROW_BUILD", "false")
+
+# Check if we're authorized to download (not asked an offline build).
+# (Note that cmake will still be downloaded if necessary
+#  https://arrow.apache.org/docs/developers/cpp/building.html#offline-builds)
+download_ok <- !test_mode && !env_is("TEST_OFFLINE_BUILD", "true")
+
+# This "tools/thirdparty_dependencies" path, within the tar file, might exist if
+# create_package_with_all_dependencies() was run, or if someone has created it
+# manually before running make build.
+# If you change this path, you also need to edit
+# `create_package_with_all_dependencies()` in install-arrow.R
+thirdparty_dependency_dir <- Sys.getenv("ARROW_THIRDPARTY_DEPENDENCY_DIR", "tools/thirdparty_dependencies")
+
+arrow_versioned <- paste0("arrow-", VERSION)
+# configure.win uses a different libarrow dir and and the zip is already nested
+if (on_windows) {
+  lib_dir <- "windows"
+  dst_dir <- lib_dir
+} else {
+  lib_dir <- "libarrow"
+  dst_dir <- file.path(lib_dir, arrow_versioned)
+}
+
+api_h <- file.path(lib_dir, arrow_versioned, "include/arrow/api.h")
+
+if (!test_mode && !file.exists(api_h)) {
   # If we're working in a local checkout and have already built the libs, we
   # don't need to do anything. Otherwise,
   # (1) Look for a prebuilt binary for this version
   bin_file <- src_dir <- NULL
 
-  if (!identical(Sys.getenv("ARROW_DOWNLOADED_BINARIES"), "")) {
-    bin_zip <- Sys.getenv("ARROW_DOWNLOADED_BINARIES")
-    cat(sprintf("*** Using pre-downloaded zip for libarrow binaries: %s\n", bin_zip))
+  # Keep backwards compatibility with winlibs.R
+  bin_zip <- Sys.getenv("ARROW_DOWNLOADED_BINARIES", Sys.getenv("RWINLIB_LOCAL", NA))
+
+  if (!is.na(bin_zip)) {
+    lg("Using pre-downloaded zip for libarrow binaries: %s", bin_zip)
     if (file.exists(bin_zip)) {
       bin_file <- tempfile()
       file.copy(bin_zip, bin_file)
     } else {
-      cat(sprintf("*** File not found: %s ($ARROW_DOWNLOADED_BINARIES)\n", bin_zip))
+      lg("File not found: %s ($ARROW_DOWNLOADED_BINARIES)", bin_zip)
       bin_file <- NULL
     }
   } else if (download_ok) {
@@ -757,7 +913,7 @@ if (!test_mode && !file.exists(paste0(dst_dir, "/include/arrow/api.h"))) {
     dir.create(dst_dir, showWarnings = !quietly, recursive = TRUE)
     unzip(bin_file, exdir = dst_dir)
     unlink(bin_file)
-  } else if (build_ok) {
+  } else if (build_ok && !on_windows) {
     # (2) Find source and build it
     src_dir <- find_local_source()
     if (!is.null(src_dir)) {
@@ -768,9 +924,9 @@ if (!test_mode && !file.exists(paste0(dst_dir, "/include/arrow/api.h"))) {
       ))
       build_libarrow(src_dir, dst_dir)
     } else {
-      cat("*** Proceeding without libarrow (no local source)\n")
+      exit("Proceeding without libarrow (no local source)")
     }
   } else {
-    cat("*** Proceeding without libarrow (build not authorized)\n")
+    exit("Proceeding without libarrow (build not authorized)")
   }
 }
